// RegistryManip.cpp: implementation of the CRegistryManip class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "RegistryManip.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CRegistryManip::CRegistryManip()
{
	_tcscpy (m_cstrValSep, c_strSepReg);
}

CRegistryManip::~CRegistryManip()
{

}



/////////////////////////////////////////////////////////////////////
//
//	Registry-functions
//
/////////////////////////////////////////////////////////////////////


BOOL CRegistryManip::OpenRegKey(const HKEY hKey, const TCHAR* pSubKey, HKEY& hKeyPrograms, 
						   REGSAM samDesired /* = KEY_READ*/)
{
	if (NULL == pSubKey) return FALSE;

	ULONG res;
	unsigned short pSubKeyRes = NULL;

	if (pSubKey == "")
	{

		res = ::RegOpenKeyEx(	hKey,
								NULL, 
								0,
								samDesired,
								&hKeyPrograms);
	}
	else
	{
		res = ::RegOpenKeyEx(	hKey,
								LPCTSTR(pSubKey),
								0,
								samDesired,
								&hKeyPrograms);
	}


	if (res != ERROR_SUCCESS)
	{
		return FALSE;
	}
	
	return TRUE;
}


BOOL CRegistryManip::CloseRegKey(HKEY hKey)
{
	ULONG res;

	res = ::RegCloseKey( hKey );

	if (res != ERROR_SUCCESS)
	{
		return FALSE;
	}
	
	return TRUE;
}


BOOL CRegistryManip::GetOneValueFromReg(HKEY hKey, const TCHAR* pRegKey, TCHAR *pRes, 
								   DWORD dwType /* = REG_SZ*/)
{
    TCHAR dwBuffer[1024];        // bytes to allocate for buffers
    DWORD dwBufferSize;    // size of dwBuffer
	ULONG res;
	int nVal = 0;

	if (NULL == pRes || NULL == pRegKey) return FALSE;
	_tcscpy(dwBuffer, _T(""));
    
	if (dwType == REG_SZ)
	{
		dwBufferSize = sizeof(dwBuffer);

		res = ::RegQueryValueEx( hKey,
				  			     pRegKey, 
							     NULL,
							     &dwType,
							     (LPBYTE) dwBuffer,
							     &dwBufferSize );
	}
	else if (dwType == REG_BINARY || dwType == REG_DWORD)
	{
		dwBufferSize = sizeof(nVal);

		res = ::RegQueryValueEx(   hKey,
								   pRegKey, 
								   NULL,
								   &dwType,
								   (LPBYTE) &nVal,
								   &dwBufferSize );

		if (res == ERROR_SUCCESS)
		{
			_stprintf(dwBuffer, _T("%d"), nVal);
		}
	}
	else
	{
		res = 1;
	}

	if (res != ERROR_SUCCESS)
	{
		return FALSE;
	}
	
	_tcscpy(pRes, dwBuffer);
	return TRUE;
}

BOOL CRegistryManip::GetCountOfSubKeys(HKEY hKey, unsigned long& nSubKeysCount)
{
	BOOL bRes = FALSE;

	nSubKeysCount = 0;

	int lasterror = ERROR_SUCCESS;
	TCHAR szClass[240]; 
	unsigned long ulClassLen = 240;
	unsigned long ulKeyLen = 240;
	unsigned long ulMaxKeyLen = 240, ulMaxClassLen = 240, ulVal = 240;
	unsigned long ulNamenlen = 240, ulMaxNamenlen = 240, ulSecurityDescriptor = 240;
	FILETIME filetime;

	// zuerst bekommen allgemenine Information-wieviel Parametern in diesem Verzeichnis stecken
	if ( (lasterror = ::RegQueryInfoKey(	hKey, 
											szClass, 
											&ulClassLen, 
											NULL, 
											&ulKeyLen, 
											&ulMaxKeyLen,
											&ulMaxClassLen,
											&ulVal,
											&ulNamenlen, 
											&ulMaxNamenlen, 
											&ulSecurityDescriptor,
											&filetime )) == ERROR_SUCCESS )
	{
		nSubKeysCount = ulVal;
		bRes = TRUE;
	}

	return bRes;
}

BOOL CRegistryManip::GetNameOfCurSubKey(HKEY hKey, int nIndex, TCHAR *pNameRes)
{
	if (NULL == pNameRes) return FALSE;
	
	int lasterror = ERROR_SUCCESS;
	BOOL bRes = TRUE;
	DWORD dwName = (MAX_PATH + 1) * sizeof (TCHAR);
	TCHAR chName[MAX_PATH + 1];

	_tcscpy (chName,   _T(""));

	lasterror = ::RegEnumKey(hKey, nIndex, chName, dwName);

	if (lasterror == ERROR_SUCCESS || lasterror == ERROR_MORE_DATA)
	{
		bRes = TRUE;
	}
	else
	{
		bRes = FALSE;
	}
	
	_tcscpy(pNameRes, chName);
	return bRes;
}


BOOL CRegistryManip::GetNamesOfAllSubKeys(HKEY hKeyBase, const CString& csKeyPath, 
										  CStringArray& acsKeyList)
{
	HKEY hKey = NULL;
	BOOL bRes = TRUE;
	int nIndex = 0;

	bRes = OpenRegKey(hKeyBase, LPCTSTR(csKeyPath), hKey); 
	if (bRes == TRUE)
	{
		while (bRes == TRUE)
		{
			TCHAR chName[MAX_PATH + 1];
			_tcscpy (chName, _T(""));

			bRes = GetNameOfCurSubKey(hKey, nIndex, chName);

			if (bRes == TRUE)
			{
				acsKeyList.Add( chName );

				CString sLocalPath =  csKeyPath + _T("\\") + chName;
				if ( csKeyPath == "" )
				{
					sLocalPath = chName;
				}
				
				//GetNamesOfAllSubKeys(hKeyBase, csKeyPath + _T("\\") + chName, acsKeyList);
				GetNamesOfAllSubKeys(hKeyBase, sLocalPath, acsKeyList);

			}

			nIndex++;
		}	// while (bRes == TRUE)

		// wenn die Liste einige Eintrge hat und bRes = FALSE, weil keine 
		// Unterverzeichnisse mehr zu finden sind, muss bRes auf TRUE gesetzt werden
		if (acsKeyList.GetSize() > 0 && bRes == FALSE)
		{
			bRes = TRUE;
		}

		// schliessen registry
		CloseRegKey(hKey);

	}	// if (bRes == TRUE)

	return bRes;
}


BOOL CRegistryManip::SetOneValue(HKEY hKey, const TCHAR* pKeyName, const TCHAR* pKeyVal, 
							DWORD dwType /*= REG_SZ*/)
{
	if (NULL == pKeyName || NULL == pKeyVal) return FALSE;

	BOOL bRes = FALSE;
	TCHAR strCurVal[256];
	DWORD nBufferSize = 0;
	long lResult = 0;

	_tcscpy (strCurVal, pKeyVal);
	nBufferSize = sizeof(TCHAR) * (_tcslen(strCurVal) + 1);

	if (dwType == REG_SZ)
	{
		lResult = ::RegSetValueEx(hKey, pKeyName, NULL, dwType, 
								  (BYTE*)(&strCurVal), 
								  nBufferSize  );
	}
	else if (dwType == REG_BINARY)
	{
		int nVal = atoi (pKeyVal);
		nBufferSize = sizeof (int);

		lResult = ::RegSetValueEx(hKey, pKeyName, NULL, dwType, 
								  (BYTE*)(&nVal), 
								  nBufferSize  );
	}
	else if (dwType == REG_DWORD)
	{
		DWORD nVal = atol (pKeyVal);
		nBufferSize = sizeof (DWORD);

		lResult = ::RegSetValueEx(hKey, pKeyName, NULL, dwType, 
								  (BYTE*)(&nVal), 
								  nBufferSize  );
	}

	if (lResult == ERROR_SUCCESS)
	{
		bRes = TRUE;
	}

	return bRes;
}



/////////////////////////////////////////////////////////////////////
//
//	help-functions
//
/////////////////////////////////////////////////////////////////////



void CRegistryManip::ParseKeysAndValuesStr(const CStringArray& acsKeyList, 
									  const CStringArray& acsValList,
									  CStringArray& acsStringIdVal, 
									  CStringArray& acsFlagVal,
									  const CString& cstrName1, 
									  const CString& cstrName2)
{
	CString cstrVal, cstrKeyName, cstrResStr ;
	int nCount = acsKeyList.GetSize();

	// gehen die Liste von ParameterNamen durch
	for (int i = 0; i < nCount; i++)
	{
		// ParameterName
		cstrKeyName = acsKeyList.GetAt(i);

		// entsprechende ParameterValue
		cstrVal = acsValList.GetAt(i);

		// aufgrund vom ParameterNamen speichern das String in eine oder in andere Map
		if (cstrKeyName == cstrName1)
		{
			ParseOneStr(acsStringIdVal, cstrVal, cstrKeyName);
		}
		else if (cstrKeyName == cstrName2)
		{
			ParseOneStr(acsFlagVal, cstrVal, cstrKeyName);
		}
/*
		// parsen ParameterValue-String
		int nPos = 1, nPosStart = 0;
		while (nPos > 0)
		{
			// suchen den Separator
			nPos = cstrVal.Find(m_cstrValSep, nPosStart);  

			if (nPos >= 0)
			{
				CString cstrOneVal;
				cstrOneVal = cstrVal.Mid(nPosStart, nPos - nPosStart);

				TRACE (_T("Key = %s *** Val = %s \n"), cstrKeyName, cstrOneVal);

				// Start position berechnen
				nPosStart = nPos + 1;

				// aufgrund vom ParameterNamen speichern das String in eine oder in andere Map
				if (cstrKeyName == cstrName1)
				{
					acsStringIdVal.Add( CString (cstrOneVal) );

				}
				else if (cstrKeyName == cstrName2)
				{
					acsFlagVal.Add( CString (cstrOneVal) );
				}
			}
		}
*/
		cstrResStr.Format ( _T("Index = %d *** Key = %s *** Val = %s \n"), i, cstrKeyName, cstrVal);
		TRACE0 (cstrResStr);
	}
}

void CRegistryManip::ParseOneStr(CStringArray& acsStringVal, const CString& cstrVal, 
							const CString& cstrKeyName, BOOL bStandSep/* = TRUE*/, 
							TCHAR* pStrCurSep/* = NULL*/)
{
	TCHAR	cstrValSep[10];

	if (bStandSep == FALSE)
	{
		if (NULL == pStrCurSep) return;

		_tcscpy (cstrValSep, pStrCurSep);
	}
	else
	{
		_tcscpy (cstrValSep, m_cstrValSep);
	}

	// parsen ParameterValue-String
	int nPos = 1, nPosStart = 0;
	while (nPos > 0)
	{
		// suchen den Separator
		nPos = cstrVal.Find(cstrValSep, nPosStart);  

		if (nPos >= 0)
		{
			CString cstrOneVal;
			cstrOneVal = cstrVal.Mid(nPosStart, nPos - nPosStart);

			TRACE2 (_T("ParseOneStr() : Key = %s *** Val = %s \n"), cstrKeyName, cstrOneVal);

			// Start position berechnen
			nPosStart = nPos + 1;

			acsStringVal.Add( CString (cstrOneVal) );
		}
	}
}

BOOL CRegistryManip::SeekCurStringInReg( const CString& cstrBaseVal, const CString& cstrCurVal, 
									const CString& cstrKeyName)
{
	BOOL bWasFound = FALSE;
	CStringArray acsStringVal;

	// parsen das String, um dann dort suchen zu knnen
	ParseOneStr(acsStringVal, cstrBaseVal, cstrKeyName);

	// und jetzt suchen, ob das neue String schon da ist
	int nCount = acsStringVal.GetSize();
	for (int i = 0; i < nCount; i++)
	{
		CString cstrOneVal = acsStringVal.GetAt(i); 
		if (cstrOneVal == cstrCurVal)
		{
			bWasFound = TRUE;
			break;
		}
	}

	return bWasFound;
}

BOOL CRegistryManip::ChangeOneVal(const HKEY hKey, const TCHAR* pSubKey, const TCHAR* pRegKeyMain, 
							 const TCHAR* pRegKeyToChange, int nValMain, int nValPaar, 
							 BOOL& bWasFound)
{
	if (NULL == pSubKey || NULL == pRegKeyMain || NULL == pRegKeyToChange) return FALSE;

	BOOL bRes = FALSE;
	bWasFound = FALSE;

	CStringArray acsKeyList, acsValList;
	CStringArray acsStringIdVal, acsFlagVal;
	CString cstrNameStrId = c_strStringIdReg, cstrNameFlag = c_strFlagReg;

	// zuerst prfen, ob dieses Verzeichnis (pSubKey) in registry zu finden ist
	bRes = CheckIfKeyExists(hKey, pSubKey);
	if (bRes == FALSE)
	{
		// wenn nicht - muss man zuerst diesen Key in der registry erzeugen:
		// ==================================================================

		// 1) parsen das SubKey-String, um die Mglichkeit zu haben rauszufinden, welche
		// Unterverzeichnisse schon existieren und welche noch nicht
		CString cstrNewKey; //( _T("SOFTWARE\\Irina\\Mischa\\Nora\\") ) - nur als Beispiel;
		cstrNewKey = pSubKey;
		CStringArray arAllSubKeys;
		int nLen = cstrNewKey.GetLength();
		if (nLen > 0)
		{
			if (cstrNewKey[nLen - 1] != '\\')
			{
				cstrNewKey += _T("\\");
			}
		}
		ParseOneStr(arAllSubKeys, cstrNewKey, _T("mein test"), FALSE, _T("\\"));

		// 2) und jetzt bauen das SubKey wieder langsam zusammen und fr jedes Verzeichnis
		// prfen, ob dieses schon existiert oder noch nicht
		// (z.B.: SubKey="SOFTWARE\\Irina\\Mischa\\Nora\\", dann bauen SubKeyNew so: 
		//  SubKeyNew[0] = "SOFTWARE\\", SubKeyNew[1] = "SOFTWARE\\Irina\\", 
		//  SubKeyNew[2] = "SOFTWARE\\Irina\\Mischa\\" usw.)
		CString cstrCurSubKey, cstrOldSubKey;
		int nCount = arAllSubKeys.GetSize();
		for (int i = 0; i < nCount; i++)
		{
			CString cstrCurPart = arAllSubKeys.GetAt(i); 
			cstrCurSubKey +=  cstrCurPart;
			cstrCurSubKey +=  _T("\\");

			if (i == 0)
			{
				cstrOldSubKey = cstrCurSubKey;
			}

			// prfen, ob das Verzeichnis existiert
			bRes = CheckIfKeyExists(hKey, cstrCurSubKey);
			if (bRes == FALSE)
			{
				// wenn noch nicht - erzeugen 
				bRes = CreateNewKey(HKEY_LOCAL_MACHINE, cstrOldSubKey, cstrCurPart);
			}

			cstrOldSubKey = cstrCurSubKey;
		}
	}

	// zuerst finden alle Parameter und entsprechende Werte (wenn die schon existieren)
	FindAllParamsWithValues( hKey, pSubKey, acsKeyList, acsValList);

	// parsen die und speichern in arrays
	ParseKeysAndValuesStr(acsKeyList, acsValList, acsStringIdVal, acsFlagVal, 
						  cstrNameStrId, cstrNameFlag);

	int nCount = acsStringIdVal.GetSize();
	BOOL bIsMainParamFound = FALSE;

	CString cstrValMain, cstrValPaar, cstrValFlagRes (_T(""));
	cstrValMain.Format(_T("%d"), nValMain) ;
	cstrValPaar.Format(_T("%d"), nValPaar) ;

	for (int i = 0; i < nCount; i++)
	{
		// den Wert aus der Liste fr main-Schlssel holen 
		CString cstrVal = acsStringIdVal.GetAt(i);

		// prfen, ob es um diesen Wert geht
		if (cstrVal == cstrValMain)
		{
			// wenn ja - setzen wieder alle Werte in ein String, aber vorher ersetzen 
			// das String mit dem Index = nIndex auf cstrValPaar
			for (int j = 0; j < nCount; j++)
			{
				// zuerst den aktuellen Wert holen
				CString cstrFlagVal = acsFlagVal.GetAt(j);

				// index prfen
				if (j == i)
				{
					// wenn es um das Element geht, das gendert werden soll, dann ersetzen es
					cstrValFlagRes += cstrValPaar;
				}
				else
				{
					// wenn nicht, dann nehmen das aktuelle Element
					cstrValFlagRes += cstrFlagVal;
				}

				// + Trennzeichen
				cstrValFlagRes += c_strSepReg;
			}

			bIsMainParamFound = TRUE;
			break;

		}	// if (cstrVal == cstrValMain)
	}	// for (int i = 0; i < nCount; i++)

	// wenn bis jetzt noch kein Eintrag zu finden ist oder der Wert vom Hauptparameter noch nicht da ist:
	if (nCount == 0 || bIsMainParamFound == 0)
	{
		BOOL bIsInReg = FALSE;
		bRes = InputNewValue(hKey, pSubKey, pRegKeyMain, nValMain, bIsInReg);

		// wenn Key eingetragen wurde und dieser Wert vorher noch nicht da war
		if (bRes == TRUE && bIsInReg == FALSE)
		{
			bRes = InputNewValue(hKey, pSubKey, pRegKeyToChange, nValPaar, bIsInReg);
		}
	}
	else
	{
		// jetzt speichern das genderte String in registry:
		HKEY hKeyPrograms;      // handle to registry key

		// open registry
		bRes = OpenRegKey(hKey, pSubKey, hKeyPrograms, KEY_ALL_ACCESS); 
		if (bRes == TRUE)
		{
			// lesen ein Parameter aus
			bRes = SetOneValue(hKeyPrograms, pRegKeyToChange, cstrValFlagRes);

			// schliessen registry
			CloseRegKey(hKeyPrograms);
		}
	}

	bWasFound = TRUE;

	// entleeren alle arrays
	acsKeyList.RemoveAll(); 
	acsValList.RemoveAll();
	acsStringIdVal.RemoveAll(); 
	acsFlagVal.RemoveAll();

	return bRes;
}


BOOL CRegistryManip::getInfoFromRegistry(const HKEY hKey, const TCHAR* pSubKey, 
									const TCHAR* pRegKey, TCHAR *pRes, 
									DWORD dwType /* = REG_SZ*/)
{
	if (NULL == pRes || NULL == pRegKey) return FALSE;

    HKEY hKeyPrograms;      // handle to registry key
    TCHAR dwBuffer[1024];        // bytes to allocate for buffers
	BOOL bRes = FALSE;

	_tcscpy (dwBuffer, _T(""));

	// open registry
	bRes = OpenRegKey(hKey, pSubKey, hKeyPrograms); 
	if (bRes == TRUE)
	{
		// lesen ein Parameter aus
		bRes = GetOneValueFromReg(hKeyPrograms, pRegKey, dwBuffer, dwType);

		// schliessen registry
		CloseRegKey(hKeyPrograms);
	}

	// kopieren Ergebnis
    _tcscpy(pRes, dwBuffer);

	return bRes;
}



void CRegistryManip::FindAllParamsWithValues(HKEY hKeyBase, const CString& csKeyPath, 
											 CStringArray& acsKeyList, 
											 CStringArray& acsValList)
{
	HKEY hKey = NULL;
	BOOL bRes = FALSE;

	// open registry
	bRes = OpenRegKey(hKeyBase, LPCTSTR(csKeyPath), hKey); 
	if (bRes == TRUE)
	{
		TRACE1(_T("CRegistryManip::FindAllParamsWithValues found %s\n"),LPCTSTR(csKeyPath));

		int lasterror = ERROR_SUCCESS;
		int nIndex = 0;

		// damit Anzahl von Parametern holen
		unsigned long ulVal = 0;
		bRes = GetCountOfSubKeys(hKey, ulVal);

		// zuerst bekommen allgemenine Information-wieviel Parametern in diesem Verzeichnis stecken
		if (bRes == TRUE)
		{
			// ulVal - Anzahl von Parametern => gehen alle diese Eintrge durch und lesen 
			// ParameterNamen und entsprechende Werte aus
			for (nIndex = 0; nIndex < ulVal; nIndex++)
			{
				DWORD dwName = (MAX_PATH + 1) * sizeof (TCHAR);
				DWORD dwData = (MAX_PATH + 1) * sizeof (TCHAR);
				TCHAR chName[MAX_PATH + 1], chData[MAX_PATH];

				_tcscpy (chName, _T(""));
				_tcscpy (chData, _T(""));

				// so kann man aktuelles (aufgrund vom nIndex) Paar 
				// "ParamName-ParamValue" auslesen
				// das Problem hier - wenn die Daten vom Typ BINARY sind, dann
				// kann man den Wert nicht richtig auslesen 
				if ( lasterror = RegEnumValue(	hKey, 
												nIndex, 
												chName,		// Name wird in chName abgelegt 
												&dwName, 
												NULL, 
												REG_NONE, 
												(LPBYTE)chData,  // Wert wird in chData abgelegt
												&dwData) == ERROR_SUCCESS )

				{
					acsKeyList.Add( chName );
					
					TRACE3 ( _T(" Index = %d *** Key = %s *** Val = %s \n"), nIndex, chName, chData);

					VALENT val_list[1];
					val_list[0].ve_valuename = chName;

					DWORD nNum_vals = sizeof(val_list) / sizeof(VALENT); 
					TCHAR cstrValueBuf[256];
					DWORD nTotsize = sizeof(cstrValueBuf) / sizeof(TCHAR);

					// holen volle Information ber das Parameter
					lasterror = RegQueryMultipleValues(
						  hKey,         // handle to key to query
						  val_list,		// pointer to array of value entry structures
						  nNum_vals,    // size of array of value entry structures
						  cstrValueBuf, // pointer to buffer for value information
						  &nTotsize		// pointer to size of value buffer
					);

					if ( lasterror == ERROR_SUCCESS)
					{
						// lesen ein Parameter aus (der Typ ist jetzt bekannt, was dazu fhrt, 
						// dass der Wert richtig rausgelesen werden kann)
						bRes = GetOneValueFromReg(hKey, chName, chData, val_list[0].ve_type);
						acsValList.Add( chData );
					}

				}	// if ( lasterror = RegEnumValue
			}	// for (nIndex = 0; nIndex < ulVal; nIndex++)
		}	// if (bRes == TRUE)
		else
		{
			if ( lasterror != ERROR_NO_MORE_ITEMS )
			{
				TRACE2(_T("Error %d while enumerating the entries in the registry %s!\n"),lasterror,LPCTSTR(csKeyPath));
			}
		}

		// schliessen registry
		CloseRegKey(hKey);
	}
}



BOOL CRegistryManip::CreateNewKey(const HKEY hKeyBase, const TCHAR* pKeyPath,  
								  const TCHAR* pNewKey)
{
	if (NULL == pKeyPath || NULL == pNewKey) return FALSE;

	BOOL bRet = FALSE;
	HKEY hKey = NULL;

	TCHAR cstrRegPath[256], cstrSep[10];

	_tcscpy (cstrSep, _T("\\"));
	_stprintf(cstrRegPath, _T("%s%s%s"), pKeyPath, cstrSep, pNewKey);

	long lResult = ::RegCreateKeyEx(hKeyBase, cstrRegPath,
									0, _T(""), 0, KEY_ALL_ACCESS, NULL, 
									&hKey, NULL);

	if (lResult == ERROR_SUCCESS)
	{
		bRet = TRUE;
	}

	// schliessen registry
	CloseRegKey(hKey);

	return bRet;
}


BOOL CRegistryManip::InputNewValue(const HKEY hKeyBase, const TCHAR* pKeyPath,  
							  const TCHAR* pKeyName, int nVal, BOOL& bIsInReg, 
							  DWORD dwType /* = REG_SZ*/)
{
	if (NULL == pKeyPath || NULL == pKeyName) return FALSE;

	BOOL bRet = FALSE; 
	HKEY hKey = NULL;
	bIsInReg = FALSE;

	// open registry
	bRet = OpenRegKey(hKeyBase, pKeyPath, hKey, KEY_ALL_ACCESS); 
	if (bRet == TRUE)
	{
		TCHAR strOldVal[256], strRes[256], strCurVal[256];

		_tcscpy (strOldVal, _T(""));
		_tcscpy (strRes, _T(""));
		_stprintf(strCurVal, _T("%d"), nVal);

		// nur fr die String-Werte
		if (dwType == REG_SZ)
		{
			// zuerst holen uns alle WErte, die schon da sind (wenn berhaupt)
			bRet = GetOneValueFromReg(hKey, pKeyName, strOldVal);
			if (bRet == FALSE)
			{
				_tcscpy (strOldVal, _T(""));
			}

			if (_tcslen(strOldVal) > 0)
			{
				// wenn ja - zuerst prfen, ob dieser Wert schon da ist
				bIsInReg = SeekCurStringInReg(strOldVal, strCurVal, pKeyName); 

				if (FALSE == bIsInReg)
				{
					// wenn nicht gefunden - addieren den neuen Wert dazu
					_stprintf(strRes, _T("%s%s%s"), strOldVal, strCurVal, m_cstrValSep);
				}
			}
			else
			{
				// wenn bis jetzt noch keine Werte da sind, dann wird unser aktueller Wert das erste
				_tcscpy (strRes, strCurVal);
				_tcscat(strRes, m_cstrValSep);
			}
		}	// if (dwType == REG_SZ)
		else
		{
			_tcscpy (strRes, strCurVal);
		}

		// wenn der Wert neu ist
		if (FALSE == bIsInReg)
		{
			// speichern den Wert
			bRet = SetOneValue(hKey, pKeyName, strRes, dwType);  
		}

		// schliessen registry
		CloseRegKey(hKey);
	}

	return bRet;
}


BOOL CRegistryManip::CheckIfKeyExists(const HKEY hKey, const TCHAR* pSubKey)
{
	if (NULL == pSubKey) return FALSE;

	// jetzt prfen, ob unser key in registry zu finden ist:
	HKEY hKeyPrograms;      // handle to registry key

	// open registry
	BOOL bRes = OpenRegKey(hKey, pSubKey, hKeyPrograms, KEY_QUERY_VALUE); 
	if (bRes == TRUE)
	{
		// schliessen registry
		CloseRegKey(hKeyPrograms);

		return TRUE;
	}

	return FALSE;
}

BOOL CRegistryManip::DeleteInfoFromRegistry(const HKEY hKey, const char* pSubKey, 
											const TCHAR* pKeyName)
{
	if (NULL == pSubKey) return FALSE;

	BOOL bRes = FALSE;
	BOOL bDelSubKey = FALSE;
	
	// wenn ParamName=NULL, muss man den SubKey lschen
	if (pKeyName == NULL) bDelSubKey = TRUE;

	if (bDelSubKey == FALSE)
	{
		// wenn es um das Parameter geht:
		bRes = DeleteParamFromRegistry( hKey, pSubKey, pKeyName ); 
	}
	else
	{
		// wenn es um SubKey geht
		bRes = DeleteSubKeyFromRegistry( hKey, pSubKey ); 
	}

	return bRes;
}

BOOL CRegistryManip::DeleteSubKeyFromRegistry(const HKEY hKey, const char* pSubKey)
{
	BOOL bRes = FALSE;
	ULONG res = 1;

	res = RegDeleteKey ( hKey, pSubKey);

	if (ERROR_SUCCESS == res)
	{
		bRes = TRUE;
	}
	else
	{
		bRes = FALSE;
	}

	return bRes;

/*
	// noch eine Mglichkeit !!!

#include <shlwapi.h>
            #pragma comment(lib,"shlwapi.lib")
            ...
            SHDeleteKey(HKEY_LOCAL_MACHINE, "Some key"); 
*/
}


BOOL CRegistryManip::DeleteParamFromRegistry(const HKEY hKey, const char* pSubKey, 
											 const TCHAR* pKeyName)
{
	BOOL bRes = FALSE;
	HKEY hKeyPrograms = NULL;      // handle to registry key
	ULONG res = 1;

	// jetzt prfen, ob unser Parameter in registry zu finden ist
	bRes = OpenRegKey(hKey, pSubKey, hKeyPrograms, KEY_SET_VALUE ); 
	if (bRes == TRUE)
	{
		res = RegDeleteValue ( hKeyPrograms, pKeyName);

		if (ERROR_SUCCESS == res)
		{
			bRes = TRUE;
		}
		else
		{
			bRes = FALSE;
		}

		// schliessen registry
		CloseRegKey(hKeyPrograms);
	}

	return bRes;
}

BOOL CRegistryManip::GetCountOfSubKeysAll(const HKEY hKey, const TCHAR* pSubKey, 
										  unsigned long& nSubKeysCount)
{
	BOOL bRes = FALSE;
	HKEY hKeyPrograms = NULL;      // handle to registry key
	ULONG res = 1;

	// jetzt prfen, ob unser Parameter in registry zu finden ist
	bRes = OpenRegKey(hKey, pSubKey, hKeyPrograms, KEY_READ ); 
	if (bRes == TRUE)
	{
		res = GetCountOfSubKeys ( hKeyPrograms, nSubKeysCount);

		if (ERROR_SUCCESS == res)
		{
			bRes = TRUE;
		}
		else
		{
			bRes = FALSE;
		}

		// schliessen registry
		CloseRegKey(hKeyPrograms);
	}

	return bRes;
}

